using System;
using System.Collections.Generic;
using System.Runtime.CompilerServices;
using Jypeli;
using Jypeli.Assets;
using Jypeli.Controls;
using Jypeli.Widgets;
using System.Threading.Tasks;
using Timer = Jypeli.Timer;

namespace Kolikkogoblin;
/// @author  Janne Yli-Tainio
/// @version 18.10.2023
/// <summary>
/// Goblin pyrkii keräämään kaikki kolikot
/// </summary>
public class Kolikkogoblin : PhysicsGame
{
    private static readonly String[] taso0 = {
                  "    EP                  ",
                  "                        ",
                  "    *                 M ",
                  "    X   *       *   XXXX",
                  "        XX      X       ",
                  "             X          ",    
                  "       X                ",
                  "    *                   ",
                  "    X                   ",
                  "                        ",
                  "      X   X    XX  *    ",
                  "                   X    ",
                  "                 X      ",
                  "   *           X        ",
                  "  XXX        X          ",
                  "        X               ",
                  "      WWWW    G         ",
                  };
    
    private static readonly String[] taso1 = {
        "                        ",
        "                        ",
        "    EP                  ",
        "                        ",
        "                      M ",
        "  *            X  X XXXX",
        "  X X  *   XX           ",
        "       X                ",    
        "                *       ",
        "    X           X       ",
        "                    ww  ",
        "wX                      ",
        "    *                   ",
        "   XXXX   X             ",
        "            X   X       ",
        "                   *    ",
        "  *                X    ",
        "  X   X         XX      ",
        "  WWW  X  W  G    WWW   ",
    };
    
    private static readonly String[] taso2 = {
        "    EP                  ",
        "                        ",
        "                        ",
        "                        ",
        "                        ",
        "*                     M ",
        "X    XXX  w X  X   XwXXX",
        " X                      ",
        "  wX                    ",    
        "    *                   ",
        "  wwX X  XXX  X   X     ",
        "                        ",
        "                     X  ",
        "    *              *    ",
        "   XXX   XXX   X   X   w",
        "                        ",
        "  X                     ",
        " *                      ",
        " XX                 *   ",
        "   X   X    X      XX   ",
        "   WWWWWWW  W G W  WWWWW",
    };

    private static readonly String[] pelilapi =
    {
        "                        ",
        "                        ",
        "                        ",
        "                        ",
        "                        ",
        "                        ",
        "                        ",
    };
    
    private static readonly int TileWidth = 800 / taso0[0].Length;
    private static readonly int TileHeight = 480 / taso0.Length;
    private static readonly string[][] tasolista = { taso0, taso1, taso2, pelilapi};
    
    private IntMeter KeratytKolikot;
    private IntMeter Elamat;
    private readonly IntMeter tasoNr = new IntMeter(0, 0, 3);

    private SoundEffect olioonsattuu = LoadSoundEffect("hitHurt");
    private SoundEffect hyppyAani = LoadSoundEffect("jump");
    private SoundEffect kolikkoaani = LoadSoundEffect("PickupCoin");
    private SoundEffect seuraavakarttaaani = LoadSoundEffect("powerUp");
    
    Image taustakuva = LoadImage("tasonkuva1");
    
    public override void Begin()
    {
        MultiSelectWindow alkuvalikko = new MultiSelectWindow("Pelin alkuvalikko", "Aloita peli","Lopeta");
        Add(alkuvalikko);
        alkuvalikko.AddItemHandler(0, AloitaUusiPeli);
        alkuvalikko.AddItemHandler(1, Exit);
        
    }
    /// <summary>
    /// Aloitetaan peli. Aluksi siivotaan kaikki, jotta voidaan aloittaa uusi peli.
    /// Tätä pitää kutsua uuden pelin aloittamiseksi.
    /// </summary>
    public void AloitaUusiPeli()
    {
        tasoNr.Value = 0;
        UusiTaso();
    }
    /// <summary>
    /// Aloitetaan pelissä uusi taso
    /// </summary>
    public void UusiTaso()
    {
        ClearAll();
        if (tasoNr.Value == 3)
        {
            NaytaIlmoitus("Onneksi Olkoon läpäisit kaikki kartat", Color.Yellow);
            return;
        }
        ClearAll();
        int index = tasoNr.Value;
        string[] tasonKuva = tasolista[index];
        Level.Background.Image = taustakuva;
        
        TileMap tiles = TileMap.FromStringArray(tasonKuva);
        tasoNr.Value++;
        Gravity = new Vector(0, -500);
        IsFullScreen = true;

        tiles.SetTileMethod('X', LuoPalikka, Color.Wheat);
        tiles.SetTileMethod('*', LuoKolikko, Color.Yellow);
        tiles.SetTileMethod('G', LuoGoblin, Color.Green);
        tiles.SetTileMethod('W', LuoPiikkeja, Color.DarkGray);
        tiles.SetTileMethod('M', LuoMaali, Color.Blue);
        tiles.SetTileMethod('P', LuoPistelaskuri, Color.Blue);
        tiles.SetTileMethod('E', LuoElamaLaskuri, Color.Blue);
        tiles.SetTileMethod('w', LuoLeijuviaPiikkeja, Color.Blue);
            

        tiles.Insert(TileWidth, TileHeight);
        tiles.Execute(TileWidth, TileHeight);
            
        Level.CreateBorders();
        Camera.ZoomToLevel();
        PhoneBackButton.Listen(ConfirmExit, "Lopeta peli");
        Keyboard.Listen(Key.Escape, ButtonState.Pressed, ConfirmExit, "Lopeta peli");
    }
    
/// <summary>
/// Luo Palikan
/// </summary>
/// <param name="paikka">Kartassa määritetyt paikat joihin luodaan palikka</param>
/// <param name="leveys"></param>
/// <param name="korkeus"></param>
/// <param name="vari"> palikan väri mikäli sille ei olisi valittu kuvaa</param>
    private void LuoPalikka(Vector paikka, double leveys, double korkeus, Color vari)
    {
        PhysicsObject palikka = new PhysicsObject(leveys * 0.95 , korkeus / 2);
        palikka.Position = paikka;
        palikka.Color = vari;
        palikka.Tag = "rakenne";
        palikka.Image = LoadImage("palikka1");
        palikka.IgnoresPhysicsLogics = true;
        palikka.Mass = 100000;
        Add(palikka);
    }
/// <summary>
/// Luo kolikon, joita keräämällä voi kulkea portista
/// </summary>
/// <param name="paikka">Kartassa määritetyt paikat joihin luodaan kolikko</param>
/// <param name="leveys"></param>
/// <param name="korkeus"></param>
/// <param name="vari"></param>
    private void LuoKolikko(Vector paikka, double leveys, double korkeus, Color vari)
    {
        PhysicsObject kolikko = new PhysicsObject(leveys / 2, leveys / 2, Shape.Circle);
        kolikko.Position = paikka;
        kolikko.Color = vari;
        kolikko.Tag = "kolikko2";
        kolikko.Image = LoadImage("kolikko2");
        kolikko.IgnoresGravity = true;
        Add(kolikko);
    }
    /// <summary>
    /// Luo goblinin eli pelaajan
    /// </summary>
    /// <param name="paikka">Kartassa määritetty paikka johon luodaan pelaaja</param>
    /// <param name="leveys"></param>
    /// <param name="korkeus"></param>
    /// <param name="vari"></param>
    private void LuoGoblin(Vector paikka, double leveys, double korkeus, Color vari)
    { 
        PlatformCharacter goblin = new PlatformCharacter(leveys * 0.8 , korkeus *1.5);
        goblin.Position = paikka;
        goblin.Color = vari;
        goblin.Image = LoadImage("goblinseisoo1");
        Add(goblin);
        
        Image[] goblinKavely = LoadImages( "goblinkavelee1","goblinkavelee1","goblinkavelee1","goblinkavelee1", "goblinkavelee2","goblinkavelee2","goblinkavelee2","goblinkavelee2", "goblinkavelee1","goblinkavelee1","goblinkavelee1","goblinkavelee1","goblinkavelee3", "goblinkavelee3","goblinkavelee3", "goblinkavelee3");
        Image goblinHyppy = LoadImage( "goblinhyppaa");
        Image goblinPaikallaan = LoadImage( "goblinseisoo1");
        
        goblin.AnimWalk = new Animation(goblinKavely);
        goblin.AnimIdle = new Animation(goblinPaikallaan);
        goblin.AnimJump = new Animation(goblinHyppy);
        
        Keyboard.Listen(Key.Left, ButtonState.Down, LiikutaPelaajaaVasemmalle, null, new Vector(-30000, 0), goblin);
        Keyboard.Listen(Key.Right, ButtonState.Down, LiikutaPelaajaaOikealle, null, new Vector(30000, 0), goblin);
        Keyboard.Listen(Key.Up, ButtonState.Down, PelaajaHyppaa, null, new Vector(0, 2200), goblin);
        Keyboard.Listen(Key.Down, ButtonState.Down, LiikutaPelaajaaAlas, null, new Vector(0, -5000), goblin);
        
        AddCollisionHandler<PlatformCharacter, PhysicsObject>(goblin, LoytaaKolikon);
        AddCollisionHandler<PlatformCharacter, PhysicsObject>(goblin, LoytaaPortaalin);
        AddCollisionHandler<PlatformCharacter, PhysicsObject>(goblin, OsuuPiikkeihin);
        
    }
    // Ohjelman avulla saadaan pelaaja hyppäämään
    private void PelaajaHyppaa(Vector vektori, PlatformCharacter goblin)
    {
        goblin.Jump(110);
    }
    // Pelaaja liikkuu oikealle
    private void LiikutaPelaajaaOikealle(Vector vektori, PlatformCharacter goblin)
    {
        goblin.Walk(80);
    }
    // Pelaaja liikkuu Vasemmalle
    private void LiikutaPelaajaaVasemmalle(Vector vektori, PlatformCharacter goblin)
    {
        goblin.Walk(-80);
    }
    // Pelaaja nopeuttaa laskeutumista
    private void LiikutaPelaajaaAlas(Vector vektori, PlatformCharacter goblin)
    {
        goblin.Push(vektori);
    }
    
/// <summary>
/// Kolikon löytämisestä seuraavat asiat
/// </summary>
/// <param name="goblin">pelaaja</param>
/// <param name="kohde">kolikko tarkemmin mikä tahansa physichsobject, mutta ohjelman sisällä tarkennetaan nimenomaan kolikkoon</param>
    private void LoytaaKolikon(PlatformCharacter goblin, PhysicsObject kohde)
    {
        if (kohde.Tag.ToString() == "kolikko2")
        {
            AddCollisionHandler(kohde, goblin, CollisionHandler.DestroyObject);
            KeratytKolikot.Value += 1;
            kolikkoaani.Play();
        }
    }
    /// <summary>
    /// Portaaliin pääsystä seuraavat asiat
    /// </summary>
    /// <param name="goblin">pelaaja</param>
    /// <param name="kohde">portaali tarkemmin mikä tahansa physichsobject, mutta ohjelman sisällä tarkennetaan nimenomaan portaaliin</param>
    private void LoytaaPortaalin(PlatformCharacter goblin, PhysicsObject kohde)
    {
        if (kohde.Tag.ToString() == "Maali")
        {
            if (KeratytKolikot.Value == KeratytKolikot.MaxValue)
            {
                MessageDisplay.Add("läpäisit kartan");
                seuraavakarttaaani.Play();
                UusiTaso();
                return;
            }
        }
    }
/// <summary>
/// Mitä tapahtuu kun pelaaja osuu piikkeihin. Pelaaja ottaa yhden iskun elämiinsä.
/// </summary>
/// <param name="goblin">pelaaja</param>
/// <param name="kohde">piikit tarkemmin mikä tahansa physichsobject, mutta ohjelman sisällä tarkennetaan nimenomaan piikkeihin</param>
    private void OsuuPiikkeihin(PlatformCharacter goblin, PhysicsObject kohde)
    {
        if (kohde.Tag.ToString() == "piikkeja")
        {
            Elamat.Value -= 1;
            olioonsattuu.Play();
            if (Elamat.Value == 0)
            {
                MessageDisplay.Add("Kuolit");
                Remove(goblin);
            }
        }
        if (kohde.Tag.ToString() == "LeijuvatPiikit")
        {
            Elamat.Value -= 1;
            olioonsattuu.Play();
            if (Elamat.Value == 0)
            {
                MessageDisplay.Add("Kuolit");
                Remove(goblin);
            }
        }
    }
    /// <summary>
    /// Luodaan kartalle piikit
    /// </summary>
    /// <param name="paikka">paikka kartassa</param>
    /// <param name="leveys"></param>
    /// <param name="korkeus"></param>
    /// <param name="vari"></param>
    private void LuoPiikkeja(Vector paikka, double leveys, double korkeus, Color vari)
        {
            PhysicsObject piikkeja = new PhysicsObject(leveys, 30);
            piikkeja.Position = paikka;
            Image piikkienkuva = LoadImage("piikit");
            piikkeja.Image = piikkienkuva;
            piikkeja.Tag = "piikkeja";
            piikkeja.IgnoresPhysicsLogics = true;
            piikkeja.Mass = 100000;
            Add(piikkeja);

        }
    /// <summary>
    /// Luodaan kartalle piikit, jotka eivät ole pohjakerroksessa.
    /// </summary>
    /// <param name="paikka">paikka kartassa</param>
    /// <param name="leveys">taulukon yhden ruudun leveys</param>
    /// <param name="korkeus">taulukon yhden ruudun korkeus</param>
    /// <param name="vari">olion vari kuvan alla</param>
    private void LuoLeijuviaPiikkeja(Vector paikka, double leveys, double korkeus, Color vari)
    {
        PhysicsObject leijuviapiikkeja = new PhysicsObject(leveys, 30);
        leijuviapiikkeja.Position = paikka;
        Image piikkienkuva = LoadImage("LeijuvatPiikit");
        leijuviapiikkeja.Image = piikkienkuva;
        leijuviapiikkeja.Tag = "piikkeja";
        leijuviapiikkeja.IgnoresPhysicsLogics = true;
        leijuviapiikkeja.Mass = 100000;
        Add(leijuviapiikkeja);

    } 
    /// <summary>
    /// LuoMaalin eli portaalin kartalle
    /// </summary>
    /// <param name="paikka">kartassa määritetty paikka, johon luodaan portaali</param>
    /// <param name="leveys"></param>
    /// <param name="korkeus"></param>
    /// <param name="vari"></param>
    private void LuoMaali(Vector paikka, double leveys, double korkeus, Color vari)
    {
        PhysicsObject portaali = new PhysicsObject(leveys * 1.5, korkeus * 2);
        portaali.Position = paikka;
        portaali.Color = vari;
        portaali.Tag = "Maali";
        portaali.Image = LoadImage("Portti");
        portaali.IgnoresPhysicsLogics = true;
        portaali.Mass = 10000;
        Add(portaali);
    }
    
    /// <summary>
    /// Luo pistelaskurin johon lasketaan kolikkojen määrä.
    /// </summary>
    /// <param name="paikka">Kartassa määritetty paikka pistelaskurille</param>
    /// <param name="leveys"></param>
    /// <param name="korkeus"></param>
    /// <param name="vari"></param>
    private void LuoPistelaskuri(Vector paikka, double leveys, double korkeus, Color vari)
    {
        KeratytKolikot = new IntMeter(0);
        KeratytKolikot.MaxValue = 6;
        KeratytKolikot.UpperLimit += KaikkiKeratty;
        
        Label pistenaytto = new Label(); 
        pistenaytto.X = Screen.Left + 60;
        pistenaytto.Y = Screen.Top - 100;
        pistenaytto.TextColor = Color.Black;
        pistenaytto.Color = Color.Silver;
        pistenaytto.Title = "Kolikot:";

        pistenaytto.BindTo(KeratytKolikot);
        Add(pistenaytto);
    }
    // ilmoittaa kun kaikki kolikot on kerätty, että pelaaja voi kulkea portista
    private void KaikkiKeratty()
    {
        MessageDisplay.Add("Voit nyt kulkea Portista");
    }
    /// <summary>
    /// Luo elämälaskurin, jonka avulla goblin voi kuolla piikkeihin.
    /// </summary>
    /// <param name="paikka">Kartassa määritetty paikka elämälaskurille</param>
    /// <param name="leveys"></param>
    /// <param name="korkeus"></param>
    /// <param name="vari"></param>
    private void LuoElamaLaskuri(Vector paikka, double leveys, double korkeus, Color vari)
    {
        Elamat = new IntMeter(3);
        
        Label elamat = new Label(); 
        elamat.X = Screen.Left + 160;
        elamat.Y = Screen.Top - 100;
        elamat.TextColor = Color.Black;
        elamat.Color = Color.Silver;
        elamat.Title = "Health:";

        elamat.BindTo(Elamat);
        Add(elamat);
    }
     /// <summary>
     /// 
     /// </summary>
     /// <param name="teksti"></param>
     /// <param name="vari"></param>
    private void NaytaIlmoitus(string teksti, Color vari)
    {
        Label label = new Label(teksti);
        label.Width = 400;
        label.Height = 200;
        label.Color = vari;
        label.BorderColor = Color.Black;
        Add(label);
    }
}
